import io
import os
from six.moves import urllib

from PIL import Image

from jet_bridge import settings
from jet_bridge.exceptions.not_found import NotFound
from jet_bridge.media_cache import cache
from jet_bridge.views.base.api import APIView


class ImageResizeHandler(APIView):

    def create_thumbnail(self, file, thumbnail_path, max_width, max_height):
        img = Image.open(file)
        img.thumbnail((max_width, max_height), Image.ANTIALIAS)

        if not os.path.exists(os.path.dirname(thumbnail_path)):
            try:
                os.makedirs(os.path.dirname(thumbnail_path))
            except OSError:
                raise

        img.save(thumbnail_path, format=img.format, quality=85)  # TODO: determine real extension from format

    def get(self):
        # TODO: Move to serializer

        path = self.get_argument('path')
        max_width = self.get_argument('max_width', 320)
        max_height = self.get_argument('max_height', 240)
        external_path = path.startswith('http://') or path.startswith('https://')
        thumbnail_full_path = cache.full_path(path)

        try:
            if not os.path.exists(thumbnail_full_path):
                if not external_path:
                    file = os.path.join(settings.MEDIA_ROOT, path)

                    if not os.path.exists(file):
                        raise NotFound
                else:
                    fd = urllib.request.urlopen(path)
                    file = io.BytesIO(fd.read())

                cache.clear_cache_if_needed()
                self.create_thumbnail(file, thumbnail_full_path, max_width, max_height)
                cache.add_file(thumbnail_full_path)

            # self.set_header('Content-Type', 'image/jpg')
            #
            # with open(thumbnail_full_path, 'rb') as f:
            #     data = f.read()
            #     self.write(data)
            # self.finish()

            self.redirect(cache.url(path))
        except IOError as e:
            raise e



import logging

from jet_bridge.utils.backend import register_token
from jet_bridge.db import Session


def register_token_command():
    try:
        session = Session()
        token, created = register_token(session)

        if not created and token:
            logging.info('Token already exists: {}'.format(token.token))
        elif not created and not token:
            logging.info('Token creation failed')
        elif created and token:
            logging.info('Token created: {}'.format(token.token))
    finally:
        session.close()



from datetime import datetime

import logging
import requests

from jet_bridge import settings, VERSION
from jet_bridge.models.token import Token


def api_method_url(method):
    return '{}/{}'.format(settings.API_BASE_URL, method)


def get_token(session):
    token = session.query(Token).first()
    return token.token if token else None


def register_token(session):
    token = session.query(Token).first()

    if token:
        return token, False

    url = api_method_url('project_tokens/')
    headers = {
        'User-Agent': 'Jet Django'
    }
    data = {
        'bridge_type': 'jet_bridge',
        'bridge_version': VERSION
    }

    r = requests.request('POST', url, headers=headers, data=data)
    success = 200 <= r.status_code < 300

    if not success:
        logging.error('Register Token request error', r.status_code, r.reason)
        return None, False

    result = r.json()

    # TODO: add serializer
    token = result['token'].replace('-', '')
    date_add = datetime.strptime(result['date_add'][:-6], '%Y-%m-%dT%H:%M:%S.%f')

    token = Token(token=token, date_add=date_add)
    session.add(token)
    session.commit()

    return token, True


def is_token_activated(session):
    token = session.query(Token).first()

    if not token:
        return False

    url = api_method_url('project_tokens/{}/'.format(token.token))
    headers = {
        'User-Agent': 'Jet Django'
    }

    r = requests.request('GET', url, headers=headers)
    success = 200 <= r.status_code < 300

    if not success:
        return False

    result = r.json()

    return bool(result.get('activated'))


def reset_token(session):
    session.query(Token).delete()
    session.commit()

    return register_token(session)


def set_token(session, token):
    project_token = session.query(Token).first()
    token_clean = str(token).replace('-', '')

    if project_token:
        if project_token.token == token_clean:
            logging.info('This token is already set, ignoring')
            return

        project_token.token = token_clean
        project_token.date_add = datetime.now()
        session.commit()
        logging.info('Token changed to {}'.format(project_token.token))
    else:
        project_token = Token(token=token_clean, date_add=datetime.now())
        session.add(project_token)
        session.commit()
        logging.info('Token created {}'.format(project_token.token))


def project_auth(session, token, permission=None):
    project_token = session.query(Token).first()

    if not project_token:
        return {
            'result': False
        }

    url = api_method_url('project_auth/')
    data = {
        'project_token': project_token.token,
        'token': token
    }
    headers = {
        'User-Agent': 'Jet Django'
    }

    if permission:
        data.update(permission)

    r = requests.request('POST', url, data=data, headers=headers)
    success = 200 <= r.status_code < 300

    if not success:
        logging.error('Project Auth request error', r.status_code, r.reason)
        return {
            'result': False
        }

    result = r.json()

    if result.get('access_disabled'):
        return {
            'result': False,
            'warning': result.get('warning')
        }

    return {
        'result': True,
        'warning': result.get('warning')
    }



import uuid
import logging

from jet_bridge.utils.backend import set_token
from jet_bridge.db import Session


def set_token_command(args):
    try:
        session = Session()
        token = uuid.UUID(args[1]) if len(args) >= 2 else None

        if not token:
            logging.info('No token was specified')
            return

        set_token(session, token)
    finally:
        session.close()



from jet_bridge import VERSION, settings
from jet_bridge.responses.base import Response
from jet_bridge.views.base.api import APIView


class ApiHandler(APIView):

    def get(self):
        media_url_base = settings.MEDIA_BASE_URL + '{}' if settings.MEDIA_BASE_URL \
            else self.build_absolute_uri('/media/{}')
        response = Response({
            'version': VERSION,
            'type': 'jet_bridge',
            'media_url_template': media_url_base
        })
        self.write_response(response)



import logging

import sys

from jet_bridge.commands.register_token import register_token_command
from jet_bridge.commands.reset_token import reset_token_command
from jet_bridge.commands.run import run_command
from jet_bridge.commands.set_token import set_token_command
from jet_bridge.commands.token import token_command

logging.getLogger().setLevel(logging.INFO)


def main():
    args = sys.argv[1:]

    if len(args) >= 1:
        if args[0] == 'register_token':
            register_token_command()
            return
        elif args[0] == 'reset_token':
            reset_token_command()
            return
        elif args[0] == 'set_token':
            set_token_command(args)
            return
        elif args[0] == 'token':
            token_command()
            return

    run_command()

if __name__ == '__main__':
    main()



import logging

from jet_bridge.utils.backend import register_token, get_token
from jet_bridge.db import Session


def token_command():
    try:
        session = Session()
        token = get_token(session)

        if token:
            logging.info('Jet Admin Token:')
            logging.info(token)
        else:
            logging.info('Jet Admin Token is not set')
    finally:
        session.close()
